class Output(object):
    def name(self):
        pass  # Your implementation for Name goes here

    def connect(self):
        pass  # Your implementation for Connect goes here

    def close(self):
        pass  # Your implementation for Close goes here

    def write(self, metrics):
        pass  # Your implementation for Write goes here


class AggregatingOutput(Output):
    def add(self, metric):
        pass  # Your implementation for Add goes here

    def push(self):
        pass  # Your implementation for Push goes here

    def reset(self):
        pass  # Your implementation for Reset goes here
